@@ -1,12 +1,12 @@
 package JE::Code;
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 use strict;
 use warnings;
 
 #use Data::Dumper;
-use Exporter 'import';
+use Exporter 5.57 'import';
 our @EXPORT_OK = 'add_line_number';
 
 require JE::Object::Error::ReferenceError;
@@ -169,7 +169,7 @@ sub add_line_number {
 
 package JE::Code::Statement; # This does not cover expression statements.
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 use subs qw'_create_vars _eval_term';
 use List::Util 'first';
@@ -605,7 +605,7 @@ sub _create_vars {  # Process var and function declarations
 
 package JE::Code::Expression;
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 # B::Deparse showed me how to get these values.
 use constant nan => sin 9**9**9;
@@ -1267,7 +1267,7 @@ sub _eval_term {
 
 package JE::Code::Subscript;
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 sub str_val {
 	my $val = (my $self = shift)->[1];
@@ -1279,7 +1279,7 @@ sub str_val {
 
 package JE::Code::Arguments;
 
-our $VERSION = '0.017';
+our $VERSION = '0.018';
 
 sub list {
 	my $self = shift;
